import * as xstate from 'xstate';
import { ActorRefFrom, StateFrom } from 'xstate';
import * as react from 'react';

type CheckedState = "checked" | "unchecked" | "indeterminate";
type FormOptions = {
    name: string;
    required?: boolean;
};
type Context = {
    id: string;
    checkedState: CheckedState;
    disabled: boolean;
    value: string;
    form: FormOptions | null;
};
type Input = Pick<Context, "id"> & Partial<Pick<Context, "checkedState" | "disabled" | "value" | "form">>;
type Service = ActorRefFrom<typeof machine>;
type Send = Service["send"];
type State = StateFrom<typeof machine>;

declare const machine: xstate.StateMachine<Context, {
    type: "CHECK";
} | {
    type: "SET.CHECKED";
    checked: boolean;
} | {
    type: "SET.DISABLED";
    disabled: boolean;
} | {
    type: "SET.INDETERMINATE";
}, xstate.ProvidedActor, {
    type: "toggleCheckedState";
} | {
    type: "setCheckedState";
    params: {
        checkedState: CheckedState;
    };
} | {
    type: "setDisabled";
    params: {
        disabled: boolean;
    };
} | {
    type: "syncInputWithIndeterminate";
} | {
    type: "dispatchCheckedEvent";
} | {
    type: "onChange";
}, {
    type: "isDisabled";
} | {
    type: "isIndeterminate";
}, string, string, Input, xstate.NonReducibleUnknown, xstate.ResolveTypegenMeta<xstate.TypegenConstraint, {
    type: "CHECK";
} | {
    type: "SET.CHECKED";
    checked: boolean;
} | {
    type: "SET.DISABLED";
    disabled: boolean;
} | {
    type: "SET.INDETERMINATE";
}, xstate.ProvidedActor, {
    type: "toggleCheckedState";
} | {
    type: "setCheckedState";
    params: {
        checkedState: CheckedState;
    };
} | {
    type: "setDisabled";
    params: {
        disabled: boolean;
    };
} | {
    type: "syncInputWithIndeterminate";
} | {
    type: "dispatchCheckedEvent";
} | {
    type: "onChange";
}, {
    type: "isDisabled";
} | {
    type: "isIndeterminate";
}, string, string>>;

declare function connect(state: State, send: Send): {
    controlProps: Omit<react.DetailedHTMLProps<react.ButtonHTMLAttributes<HTMLButtonElement>, HTMLButtonElement>, "ref">;
    labelProps: Omit<react.DetailedHTMLProps<react.LabelHTMLAttributes<HTMLLabelElement>, HTMLLabelElement>, "ref">;
    hiddenInputProps: Omit<react.DetailedHTMLProps<react.InputHTMLAttributes<HTMLInputElement>, HTMLInputElement>, "ref">;
};

export { CheckedState, FormOptions, Send, Service, State, connect, machine };
