"use strict";
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);

// src/index.ts
var src_exports = {};
__export(src_exports, {
  connect: () => connect,
  machine: () => machine
});
module.exports = __toCommonJS(src_exports);

// src/machine.ts
var import_xstate = require("xstate");

// src/dom.ts
var dom = {
  getControlId: (context) => `checkbox::control::${context.id}`,
  getLabelId: (context) => `checkbox::label::${context.id}`,
  getHiddenInputId: (context) => `checkbox::hidden-input::${context.id}`,
  getHiddenInputEl: (context) => document.getElementById(
    dom.getHiddenInputId(context)
  )
};

// src/machine.ts
var machine = (0, import_xstate.createMachine)(
  {
    id: "Checkbox",
    context: ({ input }) => ({
      id: input.id,
      checkedState: (input == null ? void 0 : input.checkedState) ?? "unchecked",
      disabled: (input == null ? void 0 : input.disabled) ?? false,
      value: (input == null ? void 0 : input.value) ?? "on",
      form: (input == null ? void 0 : input.form) ?? null
    }),
    initial: "idle",
    entry: ["syncInputWithIndeterminate"],
    states: {
      idle: {
        on: {
          CHECK: [
            { guard: "isDisabled" },
            {
              guard: "isIndeterminate",
              actions: [
                {
                  type: "setCheckedState",
                  params: {
                    checkedState: "checked"
                  }
                },
                "syncInputWithIndeterminate",
                "dispatchCheckedEvent",
                "onChange"
              ]
            },
            {
              actions: [
                "toggleCheckedState",
                "syncInputWithIndeterminate",
                "dispatchCheckedEvent",
                "onChange"
              ]
            }
          ],
          "SET.INDETERMINATE": {
            actions: [
              {
                type: "setCheckedState",
                params: { checkedState: "indeterminate" }
              },
              "syncInputWithIndeterminate"
            ]
          },
          "SET.DISABLED": {
            actions: [
              {
                type: "setDisabled",
                params: ({ event }) => ({ disabled: event.disabled })
              }
            ]
          },
          "SET.CHECKED": {
            actions: [
              {
                type: "setCheckedState",
                params: ({ event }) => ({
                  checkedState: event.checked ? "checked" : "unchecked"
                })
              },
              "syncInputWithIndeterminate",
              "dispatchCheckedEvent",
              "onChange"
            ]
          }
        }
      }
    },
    types: {
      context: {},
      input: {},
      events: {},
      actions: {},
      guards: {}
    }
  },
  {
    actions: {
      setCheckedState: (0, import_xstate.assign)(({ action }) => ({
        checkedState: action.params.checkedState
      })),
      toggleCheckedState: (0, import_xstate.assign)(({ context }) => {
        const checkedState = context.checkedState === "checked" ? "unchecked" : "checked";
        return { checkedState };
      }),
      setDisabled: (0, import_xstate.assign)(({ action }) => ({
        disabled: action.params.disabled
      })),
      syncInputWithIndeterminate: ({ context }) => {
        const inputEl = dom.getHiddenInputEl(context);
        if (!inputEl)
          return;
        inputEl.indeterminate = context.checkedState === "indeterminate";
      },
      dispatchCheckedEvent: ({ context }) => {
        var _a;
        const checked = context.checkedState === "checked";
        const inputEl = dom.getHiddenInputEl(context);
        if (!inputEl)
          return;
        if (!(inputEl instanceof globalThis.HTMLInputElement))
          return;
        const prototype = Object.getPrototypeOf(inputEl);
        const descriptor = Object.getOwnPropertyDescriptor(
          prototype,
          "checked"
        );
        if (!descriptor)
          return;
        (_a = descriptor.set) == null ? void 0 : _a.call(inputEl, checked);
        inputEl.dispatchEvent(new globalThis.Event("click", { bubbles: true }));
      },
      // template
      onChange: () => {
      }
    },
    guards: {
      isDisabled: ({ context }) => context.disabled,
      isIndeterminate: ({ context }) => context.checkedState === "indeterminate"
    }
  }
);

// src/connect.ts
var import_properties = require("@react-dive-ui/properties");
function connect(state, send) {
  const context = state.context;
  const form = context.form;
  return {
    controlProps: import_properties.properties.button({
      id: dom.getControlId(context),
      type: "button",
      role: "checkbox",
      "aria-checked": context.checkedState === "indeterminate" ? "mixed" : context.checkedState === "checked",
      "aria-labelledby": dom.getLabelId(context),
      onClick: () => {
        send({ type: "CHECK" });
      },
      disabled: context.disabled,
      "data-state": context.checkedState,
      "data-disabled": context.disabled ? "" : void 0
    }),
    labelProps: import_properties.properties.label({
      id: dom.getLabelId(context),
      htmlFor: dom.getControlId(context),
      onClick: (ev) => {
        if (context.disabled) {
          ev.preventDefault();
        }
      },
      "data-state": context.checkedState,
      "data-disabled": context.disabled ? "" : void 0
    }),
    hiddenInputProps: import_properties.properties.input({
      type: "checkbox",
      tabIndex: -1,
      id: dom.getHiddenInputId(context),
      value: "on",
      "aria-labelledby": dom.getLabelId(context),
      checked: context.checkedState === "checked",
      name: form ? form.name : void 0,
      required: (form == null ? void 0 : form.required) ?? void 0,
      onChange: (ev) => {
        send({ type: "SET.CHECKED", checked: ev.target.checked });
      },
      "data-state": context.checkedState,
      "data-disabled": context.disabled ? "" : void 0,
      "aria-hidden": true,
      style: {
        overflow: "hidden",
        clip: "rect(0 0 0 0)",
        height: "1px",
        width: "1px",
        margin: "-1px",
        padding: 0,
        border: 0
      }
    })
  };
}
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  connect,
  machine
});
